"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Storages = void 0;
const internal_1 = require("../internal");
class Storages {
    constructor(agileInstance, config = {}) {
        this.storages = {};
        this.persistentInstances = new Set();
        this.agileInstance = () => agileInstance;
        config = internal_1.defineConfig(config, {
            localStorage: false,
            defaultStorageKey: null,
        });
        this.config = { defaultStorageKey: config.defaultStorageKey };
        if (config.localStorage)
            this.instantiateLocalStorage();
    }
    instantiateLocalStorage() {
        if (!Storages.localStorageAvailable()) {
            internal_1.LogCodeManager.log('11:02:00');
            return false;
        }
        const _localStorage = new internal_1.Storage({
            key: 'localStorage',
            async: false,
            methods: {
                get: localStorage.getItem.bind(localStorage),
                set: localStorage.setItem.bind(localStorage),
                remove: localStorage.removeItem.bind(localStorage),
            },
        });
        return this.register(_localStorage, { default: true });
    }
    register(storage, config = {}) {
        const hasRegisteredAnyStorage = internal_1.notEqual(this.storages, {});
        if (Object.prototype.hasOwnProperty.call(this.storages, storage.key)) {
            internal_1.LogCodeManager.log('11:03:00', [storage.key]);
            return false;
        }
        if (!hasRegisteredAnyStorage && config.default === false)
            internal_1.LogCodeManager.log('11:02:01');
        if (!hasRegisteredAnyStorage)
            config.default = true;
        this.storages[storage.key] = storage;
        if (config.default)
            this.config.defaultStorageKey = storage.key;
        this.persistentInstances.forEach((persistent) => {
            if (persistent.storageKeys.includes(storage.key)) {
                const isValid = persistent.validatePersistent();
                if (isValid)
                    persistent.initialLoading();
                return;
            }
            if (persistent.config.defaultStorageKey == null) {
                persistent.assignStorageKeys();
                const isValid = persistent.validatePersistent();
                if (isValid)
                    persistent.initialLoading();
            }
        });
        internal_1.LogCodeManager.log('13:00:00', [storage.key], storage);
        return true;
    }
    getStorage(storageKey) {
        if (!storageKey)
            return undefined;
        const storage = this.storages[storageKey];
        if (!storage) {
            internal_1.LogCodeManager.log('11:03:01', [storageKey]);
            return undefined;
        }
        if (!storage.ready) {
            internal_1.LogCodeManager.log('11:03:02', [storageKey]);
            return undefined;
        }
        return storage;
    }
    get(storageItemKey, storageKey) {
        if (!this.hasStorage()) {
            internal_1.LogCodeManager.log('11:03:03');
            return Promise.resolve(undefined);
        }
        if (storageKey) {
            const storage = this.getStorage(storageKey);
            if (storage)
                return storage.get(storageItemKey);
        }
        const defaultStorage = this.getStorage(this.config.defaultStorageKey);
        return ((defaultStorage === null || defaultStorage === void 0 ? void 0 : defaultStorage.get(storageItemKey)) || Promise.resolve(undefined));
    }
    set(storageItemKey, value, storageKeys) {
        var _a;
        if (!this.hasStorage()) {
            internal_1.LogCodeManager.log('11:03:04');
            return;
        }
        if (storageKeys != null) {
            for (const storageKey of storageKeys)
                (_a = this.getStorage(storageKey)) === null || _a === void 0 ? void 0 : _a.set(storageItemKey, value);
            return;
        }
        const defaultStorage = this.getStorage(this.config.defaultStorageKey);
        defaultStorage === null || defaultStorage === void 0 ? void 0 : defaultStorage.set(storageItemKey, value);
    }
    remove(storageItemKey, storageKeys) {
        var _a;
        if (!this.hasStorage()) {
            internal_1.LogCodeManager.log('11:03:05');
            return;
        }
        if (storageKeys) {
            for (const storageKey of storageKeys)
                (_a = this.getStorage(storageKey)) === null || _a === void 0 ? void 0 : _a.remove(storageItemKey);
            return;
        }
        const defaultStorage = this.getStorage(this.config.defaultStorageKey);
        defaultStorage === null || defaultStorage === void 0 ? void 0 : defaultStorage.remove(storageItemKey);
    }
    hasStorage() {
        return internal_1.notEqual(this.storages, {});
    }
    static localStorageAvailable() {
        try {
            localStorage.setItem('_myDummyKey_', 'myDummyValue');
            localStorage.removeItem('_myDummyKey_');
            return true;
        }
        catch (e) {
            return false;
        }
    }
}
exports.Storages = Storages;
